--
-- Copyright (C) 1996 Ada Resource Association (ARA), Columbus, Ohio.
-- Author: Gilles Demailly
--
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright and authorship notice appear in all
-- copies and that both that copyright notice and this permission notice
-- appear in supporting documentation.
-- 
-- The ARA makes no representations about the suitability of this software
-- for any purpose.  It is provided "as is" without express
-- or implied warranty.
-- 

-- This child package provides the class Constant_Value_Attribute :
-- Constant value attributes contain a reference to the Constant Pool array.
--
-- For more information about Java Class file format check :
--    The Java Virtual Machine Specification
--    (Release 1.0 Beta - Draft - August 21, 1995)


with Basic_Definitions;
use Basic_Definitions;

with Byte_Utilities;

with CP;

package Attribute.Constant_Value is

   -- This class attribute name in Java files
   ------------------------------------------
   C_Class_Name : constant Wide_String := "ConstantValue";
   
   -- type Constant_Value_Attribute (subclass of Attribute)
   --------------------------------------------------------
   type Constant_Value_Attribute is new Attribute with private;
      
   -- Access to Constant_Value_Attribute
   -------------------------------------
   type Acc_Constant_Value_Attribute 
     is access all Constant_Value_Attribute'Class;
     
   -- Decode informations of the attribute from a file
   ---------------------------------------------------
   procedure Decode (This      : access Constant_Value_Attribute;
                     From_File : in Byte_Utilities.File_Type;
                     Context   : in CP.Acc_CP_Infos);

   -- Display the attribute
   ------------------------
   procedure Display (This    : access Constant_Value_Attribute;
                      Context : in CP.Acc_CP_Infos);

private

   -- extends the Attribute with an Index in the Constant Pool table
   -----------------------------------------------------------------
   type Constant_Value_Attribute is new Attribute with
      record
         Index : Unsigned_16;
      end record;

end Attribute.Constant_Value;
