--
-- Copyright (C) 1996 Ada Resource Association (ARA), Columbus, Ohio.
-- Author: Gilles Demailly
--
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright and authorship notice appear in all
-- copies and that both that copyright notice and this permission notice
-- appear in supporting documentation.
-- 
-- The ARA makes no representations about the suitability of this software
-- for any purpose.  It is provided "as is" without express
-- or implied warranty.
-- 

-- This child package provides the class Line_Number_Table_Attribute :
--  this attribute is used by debuggers and the exception handler to
--  determine which part of the virtual machine code corresponds to
--  a given location in the source.
--
-- For more information about Java Class file format check :
--    The Java Virtual Machine Specification
--    (Release 1.0 Beta - Draft - August 21, 1995)


with Basic_Definitions;
use Basic_Definitions;

with Byte_Utilities;

with CP;


package Attribute.Line_Number_Table is

   -- This class attribute name in Java files
   ------------------------------------------
   C_Class_Name : constant Wide_String := "LineNumberTable";

   -- type Line_Number_Table_Attribute (subclass of Attribute)
   -----------------------------------------------------------
   type Line_Number_Table_Attribute is new Attribute with private;
      
   -- Access to Line_Number_Table_Attribute
   ----------------------------------------
   type Acc_Line_Number_Table_Attribute
     is access all Line_Number_Table_Attribute'Class;
     
   -- Decode informations of the attribute from a file
   ---------------------------------------------------
   procedure Decode (This      : access Line_Number_Table_Attribute;
                     From_File : in Byte_Utilities.File_Type;
                     Context   : in CP.Acc_CP_Infos);

   -- Display the attribute
   ------------------------
   procedure Display (This    : access Line_Number_Table_Attribute;
                      Context : in CP.Acc_CP_Infos);

private

   -- one association between a code line and a source line
   --------------------------------------------------------
   type Table_Info is
      record
         Start_Pc    : Unsigned_16;
         Line_Number : Unsigned_16;
      end record;

   type Table_Infos is array (Unsigned_16 range <>) of Table_Info;

   type Acc_Table_Infos is access Table_Infos;
   
   -- adds a table of associations between code and source lines
   -------------------------------------------------------------
   type Line_Number_Table_Attribute is new Attribute with
      record
         Line_Number_Table_Length : Unsigned_16;
         Line_Number_Table        : Acc_Table_Infos;
      end record;

end Attribute.Line_Number_Table;
