--
-- Copyright (C) 1996 Ada Resource Association (ARA), Columbus, Ohio.
-- Author: Gilles Demailly
--
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright and authorship notice appear in all
-- copies and that both that copyright notice and this permission notice
-- appear in supporting documentation.
-- 
-- The ARA makes no representations about the suitability of this software
-- for any purpose.  It is provided "as is" without express
-- or implied warranty.
-- 

-- This child package provides the class Source_File_Attribute :
--  this attribute is used to indicate the name of the source file
--  of the compiled class.
--
-- For more information about Java Class file format check :
--    The Java Virtual Machine Specification
--    (Release 1.0 Beta - Draft - August 21, 1995)

with Basic_Definitions;
use Basic_Definitions;

with Byte_Utilities;

with CP;

package Attribute.Source_File is

   C_Class_Name : constant Wide_String := "SourceFile";
   
   -- type Source_File_Attribute (subclass of Attribute)
   -----------------------------------------------------
   type Source_File_Attribute is new Attribute with private;
      
   -- Access to Source_File_Attribute
   ----------------------------------
   type Acc_Source_File_Attribute is access all Source_File_Attribute'Class;
     
   -- Decode informations of the attribute from a file
   ---------------------------------------------------
   procedure Decode (This      : access Source_File_Attribute;
                     From_File : in Byte_Utilities.File_Type;
                     Context   : in CP.Acc_CP_Infos);

   -- Display the attribute
   ------------------------
   procedure Display (This    : access Source_File_Attribute;
                      Context : in CP.Acc_CP_Infos);

private

   -- adds the index in the Constant Pool for the source file name
   ---------------------------------------------------------------
   type Source_File_Attribute is new Attribute with
      record
         Index : Unsigned_16;
      end record;

end Attribute.Source_File;
