--
-- Copyright (C) 1996 Ada Resource Association (ARA), Columbus, Ohio.
-- Author: Gilles Demailly
--
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright and authorship notice appear in all
-- copies and that both that copyright notice and this permission notice
-- appear in supporting documentation.
-- 
-- The ARA makes no representations about the suitability of this software
-- for any purpose.  It is provided "as is" without express
-- or implied warranty.
-- 


-- This child package provides the class Exceptions_Attribute :
--  this attributes stores the exceptions a method may throw.
--
-- For more information about Java Class file format check :
--    The Java Virtual Machine Specification
--    (Release 1.0 Beta - Draft - August 21, 1995)

with Basic_Definitions;
use Basic_Definitions;

with Byte_Utilities;

with CP;

package Attribute.Exceptions is

   -- This class attribute name in Java files
   ------------------------------------------
   C_Class_Name : constant Wide_String := "Exceptions";

   type Exceptions_Attribute is new Attribute with private;
      
   type Acc_Exceptions_Attribute is access all Exceptions_Attribute'Class;
     
   -- Decode informations of the attribute from a file
   ---------------------------------------------------
   procedure Decode (This      : access Exceptions_Attribute;
                     From_File : in Byte_Utilities.File_Type;
                     Context   : in CP.Acc_CP_Infos);

   -- Display the attribute
   ------------------------
   procedure Display (This    : access Exceptions_Attribute;
                      Context : in CP.Acc_CP_Infos);

private

   -- array of exception indexes in the constant pool array
   --------------------------------------------------------
   type Index_Table is array (Unsigned_16 range <>) of Unsigned_16;

   type Acc_Index_Table is access Index_Table;

   -- adds the table of exception indexes
   --------------------------------------
   type Exceptions_Attribute is new Attribute with
      record
         Number_Of_Exceptions  : Unsigned_16;
         Exception_Index_Table : Acc_Index_Table;
      end record;

end Attribute.Exceptions;
