--
-- Copyright (C) 1996 Ada Resource Association (ARA), Columbus, Ohio.
-- Author: Gilles Demailly
--
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright and authorship notice appear in all
-- copies and that both that copyright notice and this permission notice
-- appear in supporting documentation.
-- 
-- The ARA makes no representations about the suitability of this software
-- for any purpose.  It is provided "as is" without express
-- or implied warranty.
-- 

-- This child package provides the class Classes :
-- a constant pool Class information is used to represent a class
-- or an interface.
--
-- For more information about Java Class file format check :
--    The Java Virtual Machine Specification
--    (Release 1.0 Beta - Draft - August 21, 1995)

with Basic_Definitions;
use Basic_Definitions;

with Byte_Utilities;

package CP.Classes is

   -- the class Tag value in Java files
   ------------------------------------
   C_Class_Tag : constant Unsigned_8 := 7;

   type Classes is new CP_Info with private;
      
   type Acc_Classes is access all Classes'Class;
   
   -- Decode a Constant Pool information
   -------------------------------------
   procedure Decode (From_File : Byte_Utilities.File_Type;
                     Some_Info : access Classes);

   -- Display a Constant Pool information
   --------------------------------------
   procedure Display (Some_Info : access Classes;
                      Context   : in Acc_CP_Infos);

   -- Returns True is the information is a class
   -- (always return True)
   ---------------------------------------------
   function Is_Class
               (Some_Info : access Classes) return Boolean;
   
   --
   -- Differents Constant Pool information to string conversion functions
   --
   
   function Print_String
               (Some_Info : access Classes;
                Context   : in Acc_CP_Infos) return String;

   function Java_Decoded_String
               (Some_Info : access Classes;
                Context   : Acc_CP_Infos;
                Purpose   : Decoding_Purpose) return String;

private

   -- adds the class name index in the Constant Pool table
   -------------------------------------------------------
   type Classes is new CP_Info with
      record
         Name_Index : Unsigned_16;
      end record;
      
end CP.Classes;
