--
-- Copyright (C) 1996 Ada Resource Association (ARA), Columbus, Ohio.
-- Author: Gilles Demailly
--
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright and authorship notice appear in all
-- copies and that both that copyright notice and this permission notice
-- appear in supporting documentation.
-- 
-- The ARA makes no representations about the suitability of this software
-- for any purpose.  It is provided "as is" without express
-- or implied warranty.
-- 

-- This child package provides the class Long_Integers :
-- a constant pool Long_Integer information is used to represent
-- an Integer_32 constant.
--
-- For more information about Java Class file format check :
--    The Java Virtual Machine Specification
--    (Release 1.0 Beta - Draft - August 21, 1995)

with Basic_Definitions;
use Basic_Definitions;

with Byte_Utilities;

package CP.Long_Integers is

   -- the class Tag value in Java files
   ------------------------------------
   C_Class_Tag : constant Unsigned_8 := 5;

   type Long_Integers is new CP_Info with private;
      
   type Acc_Long_Integers is access all Long_Integers'Class;
   
   -- Decode a Constant Pool information
   -------------------------------------
   procedure Decode (From_File : Byte_Utilities.File_Type;
                     Some_Info : access Long_Integers);

   -- Display a Constant Pool information
   --------------------------------------
   procedure Display (Some_Info : access Long_Integers;
                      Context   : in Acc_CP_Infos);

   -- Returns True is the information uses two indexes
   -- in the Constant Pool table (which is -why?- the
   -- case for Long_Integer and Long_Floats)
   ---------------------------------------------------
   function Use_Two_Indexes_In_Table
               (Some_Info : access Long_Integers) return Boolean;

   
private

   -- adds the integer value
   -------------------------
   type Long_Integers is new CP_Info with
      record
         Integer_Value : Integer_64;
      end record;
      
end CP.Long_Integers;
