--
-- Copyright (C) 1996 Ada Resource Association (ARA), Columbus, Ohio.
-- Author: Gilles Demailly
--
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright and authorship notice appear in all
-- copies and that both that copyright notice and this permission notice
-- appear in supporting documentation.
-- 
-- The ARA makes no representations about the suitability of this software
-- for any purpose.  It is provided "as is" without express
-- or implied warranty.
-- 

-- This child package provides the class Name_And_Type :
-- a constant pool Name_And_Type information is used to represent a field
-- (a variable or a constant) or a method.
--
-- For more information about Java Class file format check :
--    The Java Virtual Machine Specification
--    (Release 1.0 Beta - Draft - August 21, 1995)

with Basic_Definitions;
use Basic_Definitions;

with Byte_Utilities;

package CP.Name_And_Type is

   -- the class Tag value in Java files
   ------------------------------------
   C_Class_Tag : constant Unsigned_8 := 12;

   type Name_And_Type is new CP_Info with private;
      
   type Acc_Name_And_Type is access all Name_And_Type'Class;
   
   -- Decode a Constant Pool information
   -------------------------------------
   procedure Decode (From_File : Byte_Utilities.File_Type;
                     Some_Info : access Name_And_Type);

   -- Display a Constant Pool information
   --------------------------------------
   procedure Display (Some_Info : access Name_And_Type;
                      Context   : in Acc_CP_Infos);

private

   -- adds indexes for the name and the signature of the constant
   --------------------------------------------------------------
   type Name_And_Type is new CP_Info with
      record
         Name_Index      : Unsigned_16;
         Signature_Index : Unsigned_16;
      end record;
   
end CP.Name_And_Type;
