--
-- Copyright (C) 1996 Ada Resource Association (ARA), Columbus, Ohio.
-- Author: Gilles Demailly
--
--
-- Permission to use, copy, modify, and distribute this software and its
-- documentation for any purpose and without fee is hereby granted,
-- provided that the above copyright and authorship notice appear in all
-- copies and that both that copyright notice and this permission notice
-- appear in supporting documentation.
-- 
-- The ARA makes no representations about the suitability of this software
-- for any purpose.  It is provided "as is" without express
-- or implied warranty.
-- 

--
-- Package Flags provides basic services for the Flags encountered
-- in Java Class files
--
-- For more information about Java Class file format check :
--    The Java Virtual Machine Specification
--    (Release 1.0 Beta - Draft - August 21, 1995)
--

with Basic_Definitions;
use Basic_Definitions;


package Flags is

   -- flags may be used with three different categories of Objects
   ---------------------------------------------------------------
   type Flag_Category is (Class_Flag, Method_Flag, Variable_Flag);

   --
   -- flag access functions
   --
   
   function Is_Public       (Flag : Unsigned_16) return Boolean;
   function Is_Private      (Flag : Unsigned_16) return Boolean;
   function Is_Protected    (Flag : Unsigned_16) return Boolean;
   function Is_Static       (Flag : Unsigned_16) return Boolean;
   function Is_Final        (Flag : Unsigned_16) return Boolean;
   function Is_Synchronized (Flag : Unsigned_16) return Boolean;
   function Is_Volatile     (Flag : Unsigned_16) return Boolean;
   function Is_Transient    (Flag : Unsigned_16) return Boolean;
   function Is_Native       (Flag : Unsigned_16) return Boolean;
   function Is_Interface    (Flag : Unsigned_16) return Boolean;
   function Is_Abstract     (Flag : Unsigned_16) return Boolean;

   -- This function is used for file checking.
   -- it returns True if the Flag corresponds to the
   -- flag category.
   -------------------------------------------------
   function Is_Correct (Flag     : Unsigned_16;
                        Category : Flag_Category) return Boolean;
   
   -- Display the flags
   --------------------
   procedure Display (Flag     : Unsigned_16;
                      Category : Flag_Category);

end Flags;
